import yaml


class Client:

    def __init__(self, config_location):
        self.config_location = config_location
        self._config = self._load_config()

    @property
    def config(self):
        return self._config

    def _load_config(self):
        with open(self.config_location, 'rt') as file:
            file_content = file.read()

        return yaml.load(file_content, Loader=yaml.SafeLoader)



import logging
import logging.config

default_logging_dict = \
    {
        "version": 1,
        "formatters": {
            "default": {
                "format": "{app_name} %(levelname)9s %(asctime)3s Module: %(module)s - Line: %(lineno)s %(message)s"
            }
        },
        "handlers": {
            "default": {
                "level": "INFO",
                "formatter": "default",
                "class": "logging.StreamHandler",
                "stream": "ext://sys.stdout"
            },
            "errors": {
                "level": "ERROR",
                "formatter": "default",
                "class": "logging.StreamHandler"
            }
        },
        "loggers": {
            "default": {
                "handlers": [
                    "default"
                ],
                "level": "INFO",
                "propagate": "no"
            }
        }
    }


class Client:

    def __init__(self, app_name):
        self._logger = self._load_logger(app_name)

    @property
    def logger(self):
        return self._logger

    @staticmethod
    def _load_logger(app_name):
        formatter = default_logging_dict['formatters']['default']['format']
        default_logging_dict['formatters']['default']['format'] = formatter.format(app_name=app_name)
        logging.config.dictConfig(default_logging_dict)
        return logging.getLogger('default')




from google.cloud import secretmanager_v1
from google.cloud.exceptions import NotFound
import json


class Client:

    def __init__(self, project, secret_name):

        self._secrets = self.get_secret(project, secret_name)

    @property
    def secrets(self):
        return self._secrets

    @staticmethod
    def get_secret(project, secret_name, version='latest'):

        client = secretmanager_v1.SecretManagerServiceClient()
        name = client.secret_version_path(project, secret_name, version)

        try:
            response = client.access_secret_version(name)
            return json.loads(response.payload.data.decode('UTF-8'))
        except NotFound:
            return None



import abc
import os

from tmg_etl_pipeline import conf
from tmg_etl_pipeline import logs
from tmg_etl_pipeline import secret_manager


class TMGETLPipeline(metaclass=abc.ABCMeta):

    def __init__(self, app_name, config_path):

        self.app_name = app_name

        self.config = conf.Client(config_path).config
        service_account = self.config.get('service_account', None)
        if service_account:
            os.environ['GOOGLE_APPLICATION_CREDENTIALS'] = service_account

        self.logger = logs.Client(app_name).logger
        self.secrets = secret_manager.Client(self.config['project'], app_name).secrets

    @abc.abstractmethod
    def run(self, *args, **kwargs):
        pass

    def cleanup(self, *args, **kwargs):
        pass

    def execute(self, *args, **kwargs):

        try:
            self.run(*args, **kwargs)
        finally:
            self.cleanup()

